//-----------------------------------------
//  Includes
//-----------------------------------------

#include <xdc/std.h>                                //mandatory - have to include first, for BIOS types
#include <ti/sysbios/BIOS.h>                        //mandatory - if you call APIs like BIOS_start()
#include <xdc/runtime/Log.h>                        //for Log_info() calls when UIA is added
#include <xdc/cfg/global.h>                         //header file for statically defined objects/handles
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/knl/Task.h>
#include <Queue.h>
#include <F2806x_GlobalPrototypes.h>
#include <F2806x_CpuTimers.h>
#include <F2806x_Examples.h>
#include <math.h>
#include "DSP28x_Project.h"                         // Device Headerfile and Examples Include File

//-----------------------------------------
// Prototypes
//-----------------------------------------
void hardware_init(void);
void InitXINT1(void);
void InitI2c(void);
void scia_fifo_init(void);     // Initialize the SCI FIFO
void scia_echoback_init(void);  // Initalize SCI for echoback
void scib_fifo_init(void);     // Initialize the SCI FIFO
void scib_echoback_init(void);  // Initalize SCI for echoback
void InitCAN(void);
void InitSciaGpio(void);
void InitScibGpio(void);
void InitECan(void);
void InitECanGpio(void);

//-----------------------------------------
// Static Globals
//-----------------------------------------

#define DATA_SIZE 70 //50
#define TID_TIMEOUT 500
#define GPS_TIMEOUT 50 //100
#define I2C_TIMEOUT 10
#define HOURS_TIMEOUT 50
#define XBEETX_TIMEOUT 50
#define SENSOR_TIMEOUT 10
#define ASSET_TIMEOUT 50
#define CAN_TIMEOUT 50
#define COORDINATOR 0x0000000000000000L
#define BROADCAST 0x000000000000FFFFL

//-----------------------------------------
// Dynamic Globals
//-----------------------------------------

unsigned long MessageReceivedCount = 0;
unsigned long MessageTransmittedCount = 0;
// long BO_counter = 0 ; //Counts the # of times node enters BO
// unsigned long long CAN_Data[10];

volatile int16_t i16ToggleCount = 0;

struct ECAN_REGS ECanaShadow;

typedef struct XBeeObj {
    unsigned char data[DATA_SIZE];                  // message value
} XBeeObj;                                  // Message object and pointer type

typedef struct GPSObj {
    unsigned char data[DATA_SIZE];                  // message value
} GPSObj;                                       // Message object and pointer type

typedef struct MsgObj {
    Queue_Elem  elem;
    unsigned char data[DATA_SIZE];
    unsigned char bytes;
} MsgObj, *Msg;             // Use Msg as pointer to MsgObj

typedef struct clock {
    unsigned int year;
    unsigned int month;
    unsigned int day;
    unsigned int hour;
    unsigned int minute;
    unsigned int second;
    unsigned int hundredthsecond;
} ClkObj;

ClkObj RealTimeClock;

unsigned char FrameHeader(Msg msgp, unsigned long long dest);
unsigned char MsgHeader(Msg msgp, unsigned char bytes);

unsigned char ReadFuelGauge(void);
signed int ReadSensor(unsigned char Register);
void EnableSensor(void);
void DisableSensor(void);
void CAN_TX_WTID(unsigned char message);

void Display(Msg msgp);
unsigned char ftoa(float num, Msg msgp, unsigned char bytes);
unsigned char uitoa(unsigned int num, Msg msgp, unsigned char bytes);
unsigned char itoa(signed int num, Msg msgp, unsigned char bytes);

void scia_xmit(int a);
void scia_msg(char * msg);
void scib_xmit(int a);
void scib_msg(char * msg);

void Engaged(Msg msgp);
void Disengaged(Msg msgp);

// unsigned long long destination = 0;
unsigned char QkCplrUnlocked = 0;
unsigned long long prevQkCplrSt = 0;
unsigned long Hours = 1;
unsigned long prevHours = 0;
unsigned char Fuel = 0;
unsigned long long prevFuel = 0;
unsigned long long CAN_test = 0;

//-----------------------------------------------------------------------------
// hardware_init()
//-----------------------------------------------------------------------------
void hardware_init(void)                    //called by main
{

// Init PLL, watchdog, periph clocks - see F2806x_SysCtrl.c file
// Clock frequency set to 90 MHz - see F2806x_Examples.h
    InitSysCtrl();

// Copy InitFlash fxn to RAM and run it - sets flash wait states for 90MHz
    memcpy(&RamfuncsRunStart,&RamfuncsLoadStart,(unsigned long)&RamfuncsLoadSize);
    InitFlash();

    // Step 2. Initalize GPIO:
    // This example function is found in the F2806x_Gpio.c file and
    // illustrates how to set the GPIO to it's default state.
    // InitGpio();
    // Setup only the GP I/O only for SCI-A and SCI-B functionality
    // This function is found in F2806x_Sci.c
    InitSciaGpio();         // Configure the the GPIO for SCIa (COMa) for the terminal console
    InitScibGpio();         // Configure the the GPIO for SCIb (COMb) for the Xbee radio

    scia_echoback_init();   // Configure the SCIa for echoback
    scia_fifo_init();       // Configure the SCIa FIFO

    scib_echoback_init();   // Configure the SCIb for echoback
    scib_fifo_init();       // Configure the SCIb FIFO

    InitI2c();              // Configure the I2C registers

    InitXINT1();            // Configure the external interrupt registers (PROX, Quick Coupler switch)

    InitCAN();              // Configure the CAN registers

// Configure GPIO34 (LD2 LED2 of Control Stick) as a GPIO output pin
    EALLOW;
    GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;
    GpioCtrlRegs.GPBDIR.bit.GPIO34 = 1;
    EDIS;

// Configure GPIO20 (PROX, Quick Coupler switch) as a GPIO output pin
    EALLOW;
    GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 0;
    GpioCtrlRegs.GPADIR.bit.GPIO20 = 1;
    EDIS;

// Configure GPIO21 (COM) as a GPIO output pin
    EALLOW;
    GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 0;
    GpioCtrlRegs.GPADIR.bit.GPIO21 = 1;
    EDIS;

// Init CPU Timers - see F2806x_CpuTimers.c for the fxn
    InitCpuTimers();

// Configure CPU-Timer 0 to interrupt every 500 milliseconds
// 90MHz CPU Freq, 500ms period (in uSec)
    ConfigCpuTimer(&CpuTimer0, 90, 500000);

// Start CPU Timer 0
    CpuTimer0Regs.TCR.all = 0x4001;

}

// Test 1,SCIA  DLB, 8-bit word, baud rate 0x0103, default, 1 STOP bit, no parity
void scia_echoback_init(void)
{
    // Note: Clocks were turned on to the SCIA peripheral
    // in the InitSysCtrl() function

    SciaRegs.SCICCR.all =0x0007;        // 1 stop bit,  No loopback
                                        // No parity,8 char bits,
                                        // async mode, idle-line protocol
    SciaRegs.SCICTL1.all =0x0003;       // enable TX, RX, internal SCICLK,
                                        // Disable RX ERR, SLEEP, TXWAKE

    SciaRegs.SCICTL2.bit.TXINTENA =1;   // Configure the TX hardware interrupt for SCIa (COMa)
    SciaRegs.SCICTL2.bit.RXBKINTENA =1; // Configure the RX hardware interrupt for SCIa (COMa)

    SciaRegs.SCIHBAUD    =0x0000;       // 115200 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).
    SciaRegs.SCILBAUD    =0x0017;       // 115200 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).

//  SciaRegs.SCIHBAUD    =0x0001;       // 9600 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).
//  SciaRegs.SCILBAUD    =0x0024;       // 9600 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).

    SciaRegs.SCICTL1.all =0x0023;       // Relinquish SCI from Reset
}

// Test 1,SCIA  DLB, 8-bit word, baud rate 0x0103, default, 1 STOP bit, no parity
void scib_echoback_init(void)
{
    // Note: Clocks were turned on to the SCIA peripheral
    // in the InitSysCtrl() function

    ScibRegs.SCICCR.all =0x0007;        // 1 stop bit,  No loopback
                                        // No parity,8 char bits,
                                        // async mode, idle-line protocol
    ScibRegs.SCICTL1.all =0x0003;       // enable TX, RX, internal SCICLK,
                                        // Disable RX ERR, SLEEP, TXWAKE

    ScibRegs.SCICTL2.bit.TXINTENA =1;   // Set the TX hardware interrupt for SCIb (COMb)
    ScibRegs.SCICTL2.bit.RXBKINTENA =1; // Set the RX hardware interrupt for SCIb (COMb)

//  ScibRegs.SCIHBAUD    =0x0000;       // 115200 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).
//  ScibRegs.SCILBAUD    =0x0017;       // 115200 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).

    ScibRegs.SCIHBAUD    =0x0001;       // 9600 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).
    ScibRegs.SCILBAUD    =0x0024;       // 9600 baud @LSPCLK = 22.5MHz (90 MHz SYSCLK).

    ScibRegs.SCICTL1.all =0x0023;       // Relinquish SCI from Reset
}

// Initalize the SCI FIFO
void scia_fifo_init(void)
{
    SciaRegs.SCIFFTX.all=0xE040;        // TXFFINT CLR = 1, TXFIFO Reset = 1, SCIFFENA = 1, SCIRST = 1
    ScibRegs.SCIFFRX.all=0x2141;        // RXFFIL4 = 1, RXFFINT CLR = 1, RXFFST = 1, RXFIFO Reset = 1
    SciaRegs.SCIFFCT.all=0x0;           // Clear the FIFO Control resgister for SCIa (COMa)
    SciaRegs.SCIFFRX.bit.RXFFIENA = 1;  // Set the FIFO RX hardware interrupt for SCIa (COMa)
    SciaRegs.SCIFFTX.bit.TXFFIENA = 1;  // Set the FIFO TX hardware interrupt for SCIa (COMa)
}

// Initalize the SCI FIFO
void scib_fifo_init(void)
{
    ScibRegs.SCIFFTX.all=0xE040;        // TXFFINT CLR = 1, TXFIFO Reset = 1, SCIFFENA = 1, SCIRST = 1
    ScibRegs.SCIFFRX.all=0x2141;        // RXFFIL4 = 1, RXFFINT CLR = 1, RXFFST = 1, RXFIFO Reset = 1
    ScibRegs.SCIFFCT.all=0x0;           // Clear the FIFO Control resgister for SCIb (COMb)
    ScibRegs.SCIFFRX.bit.RXFFIENA = 1;  // Set the FIFO RX hardware interrupt for SCIb (COMb)
    ScibRegs.SCIFFTX.bit.TXFFIENA = 1;  // Set the FIFO TX hardware interrupt for SCIb (COMb)

}

void InitSciaGpio()
{
   EALLOW;

/* Enable internal pull-up for the selected pins */
// Pull-ups can be enabled or disabled disabled by the user.
// This will enable the pullups for the specified pins.

   GpioCtrlRegs.GPAPUD.bit.GPIO28 = 0;      // Enable pull-up for GPIO28 (SCIRXDA)
   GpioCtrlRegs.GPAPUD.bit.GPIO29 = 0;      // Enable pull-up for GPIO29 (SCITXDA)
//   GpioCtrlRegs.GPAPUD.bit.GPIO7 = 0;     // Enable pull-up for GPIO7  (SCIRXDA)
//   GpioCtrlRegs.GPAPUD.bit.GPIO12 = 0;    // Enable pull-up for GPIO12 (SCITXDA)

/* Set qualification for selected pins to asynch only */
// Inputs are synchronized to SYSCLKOUT by default.
// This will select asynch (no qualification) for the selected pins.

   GpioCtrlRegs.GPAQSEL2.bit.GPIO28 = 3;    // Asynch input GPIO28 (SCIRXDA)
//   GpioCtrlRegs.GPAQSEL1.bit.GPIO7 = 3;   // Asynch input GPIO7 (SCIRXDA)

/* Configure SCI-A pins using GPIO regs*/
// This specifies which of the possible GPIO pins will be SCI functional pins.

   GpioCtrlRegs.GPAMUX2.bit.GPIO28 = 1;     // Configure GPIO28 for SCIRXDA operation
   GpioCtrlRegs.GPAMUX2.bit.GPIO29 = 1;     // Configure GPIO29 for SCITXDA operation
//   GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 2;    // Configure GPIO7  for SCIRXDA operation
//   GpioCtrlRegs.GPAMUX1.bit.GPIO12 = 2;   // Configure GPIO12 for SCITXDA operation

    EDIS;
}

void InitScibGpio()
{
       EALLOW;

    /* Enable internal pull-up for the selected pins */
    // Pull-ups can be enabled or disabled disabled by the user.
    // This will enable the pullups for the specified pins.

        GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;    // Enable pull-up for GPIO11 (SCIRXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO15 = 0;    // Enable pull-up for GPIO15 (SCIRXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO19 = 0;    // Enable pull-up for GPIO19 (SCIRXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO23 = 0;    // Enable pull-up for GPIO23 (SCIRXDB)
    //  GpioCtrlRegs.GPBPUD.bit.GPIO41 = 0;    // Enable pull-up for GPIO41 (SCIRXDB)
    //  GpioCtrlRegs.GPBPUD.bit.GPIO44 = 0;    // Enable pull-up for GPIO44 (SCIRXDB)

        GpioCtrlRegs.GPAPUD.bit.GPIO9 = 0;     // Enable pull-up for GPIO9 (SCITXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO14 = 0;    // Enable pull-up for GPIO14 (SCITXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO18 = 0;    // Enable pull-up for GPIO18 (SCITXDB)
    //  GpioCtrlRegs.GPAPUD.bit.GPIO22 = 0;    // Enable pull-up for GPIO22 (SCITXDB)
    //  GpioCtrlRegs.GPBPUD.bit.GPIO40 = 0;    // Enable pull-up for GPIO40 (SCITXDB)
    //  GpioCtrlRegs.GPBPUD.bit.GPIO58 = 0;    // Enable pull-up for GPIO58 (SCITXDB)

    /* Set qualification for selected pins to asynch only */
    // Inputs are synchronized to SYSCLKOUT by default.
    // This will select asynch (no qualification) for the selected pins.

        GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 3;  // Asynch input GPIO11 (SCIRXDB)
    //  GpioCtrlRegs.GPAQSEL1.bit.GPIO15 = 3;  // Asynch input GPIO15 (SCIRXDB)
    //  GpioCtrlRegs.GPAQSEL2.bit.GPIO19 = 3;  // Asynch input GPIO19 (SCIRXDB)
    //  GpioCtrlRegs.GPAQSEL2.bit.GPIO23 = 3;  // Asynch input GPIO23 (SCIRXDB)
    //  GpioCtrlRegs.GPBQSEL1.bit.GPIO41 = 3;  // Asynch input GPIO41 (SCIRXDB)
    //  GpioCtrlRegs.GPBQSEL1.bit.GPIO44 = 3;  // Asynch input GPIO44 (SCIRXDB)


    /* Configure SCI-B pins using GPIO regs*/
    // This specifies which of the possible GPIO pins will be SCI functional pins.

        GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 2;   // Configure GPIO11 for SCIRXDB operation
    //  GpioCtrlRegs.GPAMUX1.bit.GPIO15 = 2;   // Configure GPIO15 for SCIRXDB operation
    //  GpioCtrlRegs.GPAMUX2.bit.GPIO19 = 2;   // Configure GPIO19 for SCIRXDB operation
    //  GpioCtrlRegs.GPAMUX2.bit.GPIO23 = 3;   // Configure GPIO23 for SCIRXDB operation
    //  GpioCtrlRegs.GPBMUX1.bit.GPIO41 = 2;   // Configure GPIO41 for SCIRXDB operation
    //  GpioCtrlRegs.GPBMUX1.bit.GPIO44 = 2;   // Configure GPIO44 for SCIRXDB operation


        GpioCtrlRegs.GPAMUX1.bit.GPIO9 = 2;     // Configure GPIO9 for SCITXDB operation
    //  GpioCtrlRegs.GPAMUX1.bit.GPIO14 = 2;    // Configure GPIO14 for SCITXDB operation
    //  GpioCtrlRegs.GPAMUX2.bit.GPIO18 = 2;    // Configure GPIO18 for SCITXDB operation
    //  GpioCtrlRegs.GPAMUX2.bit.GPIO22 = 3;    // Configure GPIO22 for SCITXDB operation
    //  GpioCtrlRegs.GPBMUX1.bit.GPIO40 = 2;    // Configure GPIO40 for SCITXDB operation
    //  GpioCtrlRegs.GPBMUX2.bit.GPIO58 = 2;    // Configure GPIO58 for SCITXDB operation


        EDIS;
}

void InitI2c(void)
{
    EALLOW;
    SysCtrlRegs.PCLKCR0.bit.I2CAENCLK   = 0x0001;   // I2CAENCLK = 1: The I2C module is clocked.
    EDIS;

    EALLOW;
    GpioCtrlRegs.GPBPUD.bit.GPIO32      = 0x0000;   // Enable the interal pull-up on GPIO 32
    GpioCtrlRegs.GPBPUD.bit.GPIO33      = 0x0000;   // Enable the interal pull-up on GPIO 33
    GpioCtrlRegs.GPBQSEL1.bit.GPIO32    = 0x0003;   // Asynchronous for GPIO 32
    GpioCtrlRegs.GPBQSEL1.bit.GPIO33    = 0x0003;   // Asynchronous for GPIO 33
    GpioCtrlRegs.GPBMUX1.bit.GPIO32     = 0x0001;   // I2C data open drain bidirectional port on GPIO 32
    GpioCtrlRegs.GPBMUX1.bit.GPIO33     = 0x0001;   // I2C data open drain bidirectional port on GPIO 33

    EDIS;
    // Initialize I2C Module
    // Note :   The prescaler must be initialized only while the I2C module is in the reset state (IRS = 0 in I2CMDR).
    //          The prescaled frequency takes effect only when IRS is changed to 1.
    //          Changing the IPSC value while IRS = 1 has no effect.

    I2caRegs.I2CPSC.all = 9;                        // Prescaler - need 7-12 Mhz on module clk //6
    I2caRegs.I2CCLKL = 45;                          // NOTE: must be non zero //5
    I2caRegs.I2CCLKH = 45;                          // NOTE: must be non zero //5

//    I2caRegs.I2CIER.bit.RRDY = 1;                 // Enable RRDY interrupts
    I2caRegs.I2CIER.all = 0x0000;                   // Disable RRDY interrupts

//  I2caRegs.I2CFFTX.all    = 0x6000;               // Enable FIFO mode and TXFIFO
//  I2caRegs.I2CFFRX.all    = 0x2040;               // Enable RXFIFO, clear RXFFINT,

    I2caRegs.I2CFFTX.all    = 0x0000;               // Disable FIFO mode and TXFIFO
    I2caRegs.I2CFFRX.all    = 0x0000;               // Disable RXFIFO, clear RXFFINT,

    I2caRegs.I2CMDR.all     = 0x0020;               // 1: Take I2C out of reset
                                                    // 0: Stop I2C when suspended
}

void InitXINT1(void)
{
    // GPIO0 is an input
       EALLOW;
       GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 0;         // Use GPIO 0
       GpioCtrlRegs.GPADIR.bit.GPIO0 = 0;          // Set GPIO 0 as input
       EDIS;

    // GPIO0 is XINT1
       EALLOW;
       GpioIntRegs.GPIOXINT1SEL.bit.GPIOSEL = 0;   // Set GPIO 0 as a XINT1 (external interrupt)
       EDIS;

    // Configure XINT1
       XIntruptRegs.XINT1CR.bit.POLARITY = 3;      // Falling/Rising edge interrupt

    // Enable XINT1
       XIntruptRegs.XINT1CR.bit.ENABLE = 1;        // Enable XINT1 (external interrupt)
}

void InitCAN(void)
{
    /* Create a shadow register structure for the CAN control registers.

    This is needed, since, only 32-bit access is allowed to these registers. 16-bit access
     to these registers could potentially corrupt the register contents. This is
     especially true while writing to a bit (or group of bits) among bits 16 - 31 */

   /* Initialize the CAN module */
    InitECan();

    InitECanGpio();
    MessageReceivedCount = 0;

    /* Write to the MSGID field  */

    EALLOW;
//  ECanaMboxes.MBOX0.MSGID.all = 0x80F00400;
//  ECanaMboxes.MBOX1.MSGID.all = 0xC0F00400;
    ECanaMboxes.MBOX0.MSGID.all = 0x80EF2717;       // Transmit Read Request for Quick Coupler
    ECanaMboxes.MBOX1.MSGID.all = 0xC0EF1727;       // Recieve Read Response for Quick Coupler
    ECanaMboxes.MBOX2.MSGID.all = 0x80FDC527;       // Transmit Work Tool ID (PGN 64965/SPN 2902)

    /* Configure mask for Mailboxes*/
    ECanaLAMRegs.LAM1.all = 0xBF000000;             // MSGID Mask for MBOX1 (RX Quick Coupler): b10111111000000000000000000000000

    /* Configure bit timing parameters for eCANA*/
    ECanaShadow.CANMC.all = ECanaRegs.CANMC.all;    // Save the (CANMC) Master Control Register
    ECanaShadow.CANMC.bit.CCR = 1 ;                 // Set CCR = 1
    ECanaRegs.CANMC.all = ECanaShadow.CANMC.all;    // Set the (CANMC) Master Control Register

    // Wait until the CPU has been granted permission to change the configuration registers
    do
    {
      ECanaShadow.CANES.all = ECanaRegs.CANES.all;  // Set the (CANES) Error and Status Register
    } while(ECanaShadow.CANES.bit.CCE != 1 );       // Wait for CCE bit to be set..

    ECanaShadow.CANBTC.all = 0;
    /* The following block is for 90 MHz SYSCLKOUT. (45 MHz CAN module clock Bit rate = 1 Mbps
       See Note at end of file. */

    ECanaShadow.CANBTC.bit.BRPREG = 11;             // 2 for 1 Mbps, 5 for 500 kbps, 11 for 250 kbps, 23 for 125 kbps, 29 for 100 kbps
    ECanaShadow.CANBTC.bit.TSEG2REG = 3;            // Time Segement 2 for the Bit Timing Configuration Register (length of the phase in TQ units) TSEG2 = TSEG2reg + 1
    ECanaShadow.CANBTC.bit.TSEG1REG = 9;            // Time Segement 1 for the Bit Timing Configuration Register (length of a bit on the CAN bus) TSEG1 = TSEG1reg + 1

    ECanaShadow.CANBTC.bit.SAM = 1;                 // The CAN module samples three times and make a majority decision.
    ECanaRegs.CANBTC.all = ECanaShadow.CANBTC.all;  // Set the (CANBTC) Bit Timing Configuration Register

    ECanaShadow.CANMC.all = ECanaRegs.CANMC.all;    // Save the (CANMC) Master Control Register
    ECanaShadow.CANMC.bit.CCR = 0 ;                 // Clear CCR = 0: The CPU requests normal operation.
    ECanaShadow.CANMC.bit.ABO = 1;                  // Set ABO = 1: Auto bus on.
    ECanaRegs.CANMC.all = ECanaShadow.CANMC.all;    // Set the (CANMC) Master Control Register

    // Wait until the CPU no longer has permission to change the configuration registers
    do
    {
      ECanaShadow.CANES.all = ECanaRegs.CANES.all;  // Set the (CANES) Error and Status Register
    } while(ECanaShadow.CANES.bit.CCE != 0 );       // Wait for CCE bit to be  cleared..

    /* Configure Mailbox 0 under test as a Transmit mailbox */
    ECanaShadow.CANMD.all = ECanaRegs.CANMD.all;    // Save the (CANMD) Message Data Register
    ECanaShadow.CANMD.bit.MD0 = 0;                  // Set Mailbox 0 as transmit
    ECanaRegs.CANMD.all = ECanaShadow.CANMD.all;    // Set the (CANMD) Message Data Register

    /* Enable Mailbox 0 under test */
    ECanaShadow.CANME.all = ECanaRegs.CANME.all;    // Save the (CANME) Mailbox Enable Register
    ECanaShadow.CANME.bit.ME0 = 1;                  // Enable Mailbox 0
    ECanaRegs.CANME.all = ECanaShadow.CANME.all;    // Set the (CANME) Mailbox Enable Register

    /* Configure Mailbox 1 under test as a Recieve mailbox */
    ECanaMboxes.MBOX1.MSGID.bit.IDE = 1;            // Set Mailbox 1 with a 29-bit extended MSGID
    ECanaShadow.CANMD.all = ECanaRegs.CANMD.all;    // Save the (CANMD) Message Data Register
    ECanaShadow.CANMD.bit.MD1 = 1;                  // Set Mailbox 1 as recieve
    ECanaRegs.CANMD.all = ECanaShadow.CANMD.all;    // Set the (CANMD) Message Data Register

    /* Enable Mailbox 1 under test */
    ECanaShadow.CANME.all = ECanaRegs.CANME.all;    // Save the (CANME) Mailbox Enable Register
    ECanaShadow.CANME.bit.ME1 = 1;                  // Enable Mailbox 1
    ECanaRegs.CANME.all = ECanaShadow.CANME.all;    // Set the (CANME) Mailbox Enable Register

    /* Configure Mailbox 2 under test as a Transmit mailbox */
    ECanaShadow.CANMD.all = ECanaRegs.CANMD.all;    // Save the (CANMD) Message Data Register
    ECanaShadow.CANMD.bit.MD2= 0;                   // Set Mailbox 2 as transmit
    ECanaRegs.CANMD.all = ECanaShadow.CANMD.all;    // Set the (CANMD) Message Data Register

    /* Enable Mailbox 2 under test */
    ECanaShadow.CANME.all = ECanaRegs.CANME.all;    // Set the (CANME) Mailbox Enable Register
    ECanaShadow.CANME.bit.ME2 = 1;                  // Enable Mailbox 2
    ECanaRegs.CANME.all = ECanaShadow.CANME.all;    // Save the (CANME) Mailbox Enable Register

    /* Write to DLC field in Master Control reg */
    ECanaMboxes.MBOX0.MSGCTRL.bit.DLC = 8;          // Set the DLC to 8 Bytes for Mailbox 0
    ECanaMboxes.MBOX1.MSGCTRL.bit.DLC = 8;          // Set the DLC to 8 Bytes for Mailbox 1
    ECanaMboxes.MBOX2.MSGCTRL.bit.DLC = 8;          // Set the DLC to 8 Bytes for Mailbox 2

    EALLOW;
        ECanaRegs.CANMIL.bit.MIL1=0;                // Clear the Mailbox Interrupt Level bit
        ECanaRegs.CANGIM.bit.I0EN=1;                // Set the Global Mailbox Interrupt Mask bit
        ECanaRegs.CANMIM.all = 0x00000002;          // Set the Mailbox Interrupt Mask Register for Mailbox 1
    EDIS;
}

//-----------------------------------------
//  ======== PWM_InputTask ========
//-----------------------------------------
Void PWM_OutputTask()
{
    while(1)
    {

        System_printf("enter PWM_OutputTask()\n");

        Task_sleep(10000);

        System_printf("exit PWM_OutputTask()\n");

        System_flush(); /* force SysMin output to console */

    }
}

//-----------------------------------------
//  ======== main.c ========
//-----------------------------------------
Int main()
{ 
    hardware_init();                    // Intialize hardware via Xware

    System_printf("enter main()\n");

    DELAY_US(5000000L);                 // Delay for 5sec before entering the BIOS in order allow time to enumerate the USB

    BIOS_start();    /* does not return */

    return(0);
}
